@@ -44,12 +44,7 @@ typedef struct _AviIndex
   guint32 size;
   gboolean key;
 } AviIndex;
-typedef struct _AviMemory
-{
-  gint counter;
-  gpointer data;
-  gsize length;
-} AviMemory;
+typedef struct _AviMemory AviMemory;
 typedef struct _AviFile
 {
   FileIO *fio;                  /* ja:ファイルポインタ */
@@ -253,43 +248,36 @@ avi_file_close (AviFile *avi_file);
 * ja:AVIメモリ関数                                                            *
 *                                                                             *
 ******************************************************************************/
-/*  ja:AVIメモリを確保する
-    length,確保するバイト数
+/*  ja:AVIメモリを登録する
+      data,登録するメモリ
+    length,登録するバイト数
        RET,AVIメモリ,NULL:エラー                                            */
 AviMemory *
-avi_memory_malloc (const gsize length);
-
-
-/*  ja:AVIメモリを再確保する
-    avi_memory,AVIメモリ
-        length,確保するバイト数
-           RET,AVIメモリ,NULL:エラー                                        */
-AviMemory *
-avi_memory_realloc (AviMemory   *avi_memory,
-                    const gsize  length);
+avi_memory_register (gpointer    data,
+                     const gsize length);
 
 
 /*  ja:AVIメモリを複製する
       data,複製するメモリ
-    length,複製るバイト数
+    length,複製するバイト数
        RET,AVIメモリ,NULL:エラー                                            */
 AviMemory *
-avi_memory_dup (gconstpointer data,
-                const gsize   length);
+avi_memory_alloc (gconstpointer data,
+                  const gsize   length);
 
 
 /*  ja:AVIメモリのデータを取得する
     avi_memory,AVIメモリ
            RET,データ,NULL:エラー                                           */
-#define avi_memory_get_data(avi_memory)                                     \
-                        ((avi_memory)?((AviMemory *)(avi_memory))->data:NULL)
+gconstpointer
+avi_memory_get_data (AviMemory *avi_memory);
 
 
-/*  ja:AVIメモリのサイズを取得する
+/*  ja:AVIメモリのバイト数を取得する
     avi_memory,AVIメモリ
            RET,バイト数                                                     */
-#define avi_memory_get_size(avi_memory)                                     \
-                            ((avi_memory)?((AviMemory *)(avi_memory))->size:0)
+gsize
+avi_memory_get_length (AviMemory *avi_memory);
 
 
 /*  ja:AVIメモリの参照数を増やす
