@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,27 +35,23 @@
 void
 hide_caret (VmaidWindow *vmaid)
 {
-  gint sx;
-  GdkRectangle rc;
-
-  gtk_timeout_remove (timer_id);
+  g_source_remove (timer_id);
   timer_id = 0;
   if (caret)
     {
+      gint sx;
+
       caret = FALSE;
       sx = get_sx_frame (vmaid);
       if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
-        {
-          rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
-                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
-          rc.y = system_font_height * 2
-                        + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
-                                                        ? vmaid->height : 0);
-          rc.width = 2;
-          rc.height = vmaid->height;
-          gtk_widget_draw (vmaid->drawing, &rc);
-        }
+        gtk_widget_queue_draw_area (vmaid->drawing,
+                        (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0),
+                        system_font_height * 2
+                            + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
+                                                        ? vmaid->height : 0),
+                                                            2, vmaid->height);
     }
 }
 
@@ -70,45 +66,38 @@ draw_caret (VmaidWindow *vmaid,
   gchar *text;
   gint t, sx;
   guint context_id;
-  GdkRectangle rc;
 
   sx = get_sx_frame (vmaid);
   if (cursor && timer_id != 0 && (vmaid->cursor.frame != cursor->frame
                                     || vmaid->cursor.stream != cursor->stream))
     {
       /* ja:タイマ再設定 */
-      gtk_timeout_remove (timer_id);
+      g_source_remove (timer_id);
       timer_id = 0;
       /* ja:古いキャレットを消去 */
       if (vmaid->top <= cursor->frame && cursor->frame <= vmaid->top + sx)
-        {
-          rc.x = (cursor->frame - vmaid->top) * vmaid->width
-                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
-          rc.y = system_font_height * 2
+        gtk_widget_queue_draw_area (vmaid->drawing,
+                    (cursor->frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0),
+                    system_font_height * 2
                                     + (vmaid->avi_edit[0] && cursor->stream > 0
-                                                        ? vmaid->height : 0);
-          rc.width = 2;
-          rc.height = vmaid->height;
-          gtk_widget_draw (vmaid->drawing, &rc);
-        }
+                                                        ? vmaid->height : 0),
+                                                            2, vmaid->height);
     }
   /* ja:新しいキャレットを描画 */
   if (timer_id == 0)
     caret = TRUE;
   if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
-    {
-      rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
-                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
-      rc.y = system_font_height * 2
+    gtk_widget_queue_draw_area (vmaid->drawing,
+            (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0),
+            system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
-                                                        ? vmaid->height : 0);
-      rc.width = 2;
-      rc.height = vmaid->height;
-      gtk_widget_draw (vmaid->drawing, &rc);
-    }
+                                                        ? vmaid->height : 0),
+                                                            2, vmaid->height);
   if (timer_id == 0)
-    timer_id = gtk_timeout_add (500, signal_timeout, NULL);
+    timer_id = g_timeout_add (500, signal_timeout, NULL);
   t = (glonglong)vmaid->cursor.frame * vmaid->scale * 1000 / vmaid->rate;
   text = vmaid->select.stream < 0
         ? g_strdup_printf (_("Frame %d  Time %02d:%02d:%02d"),
@@ -252,11 +241,11 @@ clear_sel (VmaidWindow *vmaid,
           dx = (top - vmaid->top) * vmaid->width;
           gc = gdk_gc_new (vmaid->drawing->window);
           gdk_gc_set_exposures (gc, TRUE);
-          gdk_window_copy_area (vmaid->drawing->window, gc,
-                        dx > 0 ? rc.x + dx : rc.x, 0,
+          gdk_draw_drawable (vmaid->drawing->window, gc,
                         vmaid->drawing->window, dx > 0 ? rc.x : rc.x - dx, 0,
+                        dx > 0 ? rc.x + dx : rc.x, 0,
                         rc.width - ABS (dx), rc.height);
-          gdk_gc_destroy (gc);
+          g_object_unref (gc);
           if (dx < 0)
             rc.x = rc.x + rc.width + dx;
           rc.width = ABS (dx);
@@ -372,7 +361,8 @@ clear_sel (VmaidWindow *vmaid,
                                                             * vmaid->height;
           if (!vmaid->avi_edit[0])
             rc0.y -= vmaid->height;
-          gtk_widget_draw (vmaid->drawing, &rc);
+          gtk_widget_queue_draw_area (vmaid->drawing,
+                                            rc.x, rc.y, rc.width, rc.height);
           if (rc.x < 0)
             {
               rc = rc0;
@@ -441,7 +431,8 @@ clear_sel (VmaidWindow *vmaid,
         }
     }
   if (rc.x >= 0)
-    gtk_widget_draw (vmaid->drawing, &rc);
+    gtk_widget_queue_draw_area (vmaid->drawing,
+                                            rc.x, rc.y, rc.width, rc.height);
 }
 
 
@@ -455,7 +446,7 @@ set_usize (VmaidWindow *vmaid)
                         system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
                             + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height);
   gtk_widget_queue_resize (GTK_WIDGET (window));
-  gtk_widget_draw (vmaid->drawing, NULL);
+  gtk_widget_queue_draw (vmaid->drawing);
 }
 
 
