@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,23 +41,19 @@ signal_timeout (gpointer data)
   if ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0)
     {
       gint sx;
-      GdkRectangle rc;
       VmaidWindow *vmaid;
 
       vmaid = orz_mdi_get_data (ORZ_MDI (mdi), page);
       sx = get_sx_frame (vmaid);
       if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
-        {
-          rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
-                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
-          rc.y = system_font_height * 2
+        gtk_widget_queue_draw_area (vmaid->drawing,
+            (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0),
+            system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
-                                                        ? vmaid->height : 0);
-          rc.width = 2;
-          rc.height = vmaid->height;
-          gtk_widget_draw (vmaid->drawing, &rc);
-        }
+                                                        ? vmaid->height : 0),
+                                                            2, vmaid->height);
     }
 #ifdef USE_THREAD
   gdk_threads_leave ();
@@ -156,7 +152,7 @@ signal_destroy_draw (GtkWidget   *widget,
   /* ja:メモリを解放する */
   if (vmaid->timer_id != 0)
     {
-      gtk_timeout_remove (vmaid->timer_id);
+      g_source_remove (vmaid->timer_id);
       vmaid->timer_id = 0;
     }
   for (i = 0; i < 2; i++)
@@ -186,17 +182,18 @@ signal_expose_text (GtkWidget   *widget,
 
   text = g_strdup_printf (vmaid->avi_edit[0]
             && avi_frame_is_key (vmaid->avi_edit[0],pos) ? "%d*" : "%d", pos);
-  if (gdk_string_width (system_font,text) <= vmaid->width)
-    gdk_draw_string (widget->window, system_font, gc,
-                                                x, system_font_ascent, text);
+  pango_layout_set_text (system_font_layout, text, -1);
   g_free (text);
+  if (pango_layout_get_width (system_font_layout) <= vmaid->width)
+    gdk_draw_layout (widget->window, gc, x, y, system_font_layout);
   t = (glonglong)pos * vmaid->scale * 1000 / vmaid->rate;
   text = g_strdup_printf ("%02d:%02d:%02d",
                                 t / 60000 % 100, t / 1000 % 60, t / 10 % 100);
-  if (gdk_string_width (system_font, text) <= vmaid->width)
-    gdk_draw_string (widget->window, system_font, gc,
-                        x, y + system_font_ascent + system_font_height, text);
+  pango_layout_set_text (system_font_layout, text, -1);
   g_free (text);
+  if (pango_layout_get_width (system_font_layout) <= vmaid->width)
+    gdk_draw_layout (widget->window, gc, x, y + system_font_height,
+                                                        system_font_layout);
 }
 
 
@@ -425,10 +422,8 @@ signal_expose (GtkWidget      *widget,
               /* ja:選択開始位置が前のとき */
               rc0.width = vmaid->width;
               if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
-                {
-                  signal_expose_bitmap (widget, gc, avi_frame, frame,
+                signal_expose_bitmap (widget, gc, avi_frame, frame,
                                         rc0.x, rc0.y, rc0.width, rc0.height);
-                }
               rc0.x = vmaid->width;
               rc0.width = 2;
             }
@@ -572,7 +567,7 @@ signal_expose (GtkWidget      *widget,
                                         rc1.x, rc1.y, rc1.width, rc1.height);
         }
     }
-  gdk_gc_destroy (gc);
+  g_object_unref (gc);
   return TRUE;
 }
 
@@ -744,7 +739,7 @@ gboolean signal_timeout_draw (VmaidWindow *vmaid)
       if (!result)
         {
           /* ja:スクロールが終わったらタイマは削除 */
-          gtk_timeout_remove (vmaid->timer_id);
+          g_source_remove (vmaid->timer_id);
           vmaid->timer_id = 0;
         }
     }
@@ -778,8 +773,8 @@ signal_motion_notify (GtkWidget      *widget,
                             && signal_motion_notify_draw (vmaid, x, y, state)
                                                     && vmaid->timer_id == 0)
     /* ja:はじめてスクロールしたとき */
-    vmaid->timer_id
-                = gtk_timeout_add (1, (GtkFunction)signal_timeout_draw, vmaid);
+    vmaid->timer_id = g_timeout_add (1, (GSourceFunc)signal_timeout_draw,
+                                                                        vmaid);
   return TRUE;
 }
 
@@ -791,7 +786,7 @@ signal_button_release (GtkWidget      *widget,
 {
   if (vmaid->timer_id != 0)
     {
-      gtk_timeout_remove (vmaid->timer_id);
+      g_source_remove (vmaid->timer_id);
       vmaid->timer_id = 0;
     }
   return TRUE;
@@ -1137,7 +1132,7 @@ signal_key_press (GtkWidget   *widget,
               if (++page >= length)
                 page = 0;
             }
-          gtk_notebook_set_page (GTK_NOTEBOOK (mdi), page);
+          gtk_notebook_set_current_page (GTK_NOTEBOOK (mdi), page);
           gtk_widget_grab_focus (((VmaidWindow *)
                             orz_mdi_get_data (ORZ_MDI (mdi), page))->drawing);
         }
