@@ -1,6 +1,6 @@
 /*
     w32loader
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,19 +28,114 @@ extern "C" {
 #endif /* __cplusplus */
 
 
+#include "misc/fileio.h"
 /* ja:一般 */
 #define MAX_PATH 260
 #define INVALID_HANDLE_VALUE (GINT_TO_POINTER(-1))
-#define WAIT_FAILED 0xffffffff
 #define DONT_RESOLVE_DLL_REFERENCES   0x00000001
 #define LOAD_LIBRARY_AS_DATAFILE      0x00000002
 #define LOAD_WITH_ALTERED_SEARCH_PATH 0x00000008
 typedef LPVOID LPCRITICAL_SECTION;
-typedef LRESULT CALLBACK (*FARPROC)(VOID);
+typedef INT CALLBACK (*FARPROC)();
+/* ja:ウエイト */
+#define WAIT_FAILED    0xffffffff
+#define WAIT_OBJECT_0  0x00000000
+#define WAIT_ABANDONED 0x00000080
+#define WAIT_TIMEOUT   0x00000102
+#define INFINITE       0xffffffff
+/* ja:プロセス */
+#include "pshpack1.h"
+typedef struct _PROCESS_INFORMATION
+{
+  HANDLE hProcess;
+  HANDLE hThread;
+  DWORD dwProcessId;
+  DWORD dwThreadId;
+} PROCESS_INFORMATION, *PPROCESS_INFORMATION, *LPPROCESS_INFORMATION;
+typedef struct _STARTUPINFOA
+{
+  DWORD cb;
+  LPSTR lpReserved;
+  LPSTR lpDesktop;
+  LPSTR lpTitle;
+  DWORD dwX;
+  DWORD dwY;
+  DWORD dwXSize;
+  DWORD dwYSize;
+  DWORD dwXCountChars;
+  DWORD dwYCountChars;
+  DWORD dwFillAttribute;
+  DWORD dwFlags;
+  WORD wShowWindow;
+  WORD cbReserved2;
+  BYTE *lpReserved2;
+  HANDLE hStdInput;
+  HANDLE hStdOutput;
+  HANDLE hStdError;
+} STARTUPINFOA, *LPSTARTUPINFOA;
+typedef struct _STARTUPINFOW
+{
+  DWORD cb;
+  LPWSTR lpReserved;
+  LPWSTR lpDesktop;
+  LPWSTR lpTitle;
+  DWORD dwX;
+  DWORD dwY;
+  DWORD dwXSize;
+  DWORD dwYSize;
+  DWORD dwXCountChars;
+  DWORD dwYCountChars;
+  DWORD dwFillAttribute;
+  DWORD dwFlags;
+  WORD wShowWindow;
+  WORD cbReserved2;
+  BYTE *lpReserved2;
+  HANDLE hStdInput;
+  HANDLE hStdOutput;
+  HANDLE hStdError;
+} STARTUPINFOW, *LPSTARTUPINFOW;
+#include "poppack.h"
+#ifdef UNICODE
+# define STARTUPINFO STARTUPINFOW
+# define LPSTARTUPINFO LPSTARTUPINFOW
+#else /* not UNICODE */
+# define STARTUPINFO STARTUPINFOA
+# define LPSTARTUPINFO LPSTARTUPINFOA
+#endif /* not UNICODE */
 /* ja:メモリ */
-#define LMEM_INVALID_HANDLE 0x8000
+#define GMEM_FIXED          0x0000
+#define GMEM_MOVEABLE       0x0002
+#define GMEM_NOCOMPACT      0x0010
+#define GMEM_NODISCARD      0x0020
+#define GMEM_ZEROINIT       0x0040
+#define GMEM_MODIFY         0x0080
+#define GMEM_DISCARDABLE    0x0100
+#define GMEM_NOT_BANKED     0x1000
+#define GMEM_SHARE          0x2000
+#define GMEM_DDESHARE       0x2000
+#define GMEM_NOTIFY         0x4000
+#define GMEM_LOWER          GMEM_NOT_BANKED
+#define GMEM_VALID_FLAGS    0x7f72
 #define GMEM_INVALID_HANDLE 0x8000
-typedef HANDLE HGLOBAL, HLOCAL;
+#define GHND                (GMEM_MOVEABLE|GMEM_ZEROINIT)
+#define GPTR                (GMEM_FIXED|GMEM_ZEROINIT)
+#define LMEM_FIXED          GMEM_FIXED
+#define LMEM_MOVEABLE       GMEM_MOVEABLE
+#define LMEM_NOCOMPACT      GMEM_NOCOMPACT
+#define LMEM_NODISCARD      GMEM_NODISCARD
+#define LMEM_ZEROINIT       GMEM_ZEROINIT
+#define LMEM_MODIFY         GMEM_MODIFY
+#define LMEM_DISCARDABLE    GMEM_DISCARDABLE
+#define LMEM_NOT_BANKED     GMEM_NOT_BANKED
+#define LMEM_SHARE          GMEM_SHARE
+#define LMEM_DDESHARE       GMEM_DDESHARE
+#define LMEM_NOTIFY         GMEM_NOTIFY
+#define LMEM_LOWER          GMEM_NOT_BANKED
+#define LMEM_VALID_FLAGS    GMEM_VALID_FLAGS
+#define LMEM_INVALID_HANDLE GMEM_INVALID_HANDLE
+#define LHND                GHND
+#define LPTR                GPTR
+typedef LPVOID HGLOBAL, HLOCAL;
 #include "pshpack1.h"
 typedef struct _MEMORY_BASIC_INFORMATION
 {
@@ -74,6 +169,16 @@ typedef struct tagMEMORYSTATUS
 #define FILE_TYPE_CHAR           0x0002
 #define FILE_TYPE_PIPE           0x0003
 #define FILE_TYPE_REMOTE         0x8000
+#define GENERIC_READ             0x80000000
+#define GENERIC_WRITE            0x40000000
+#define FILE_SHARE_READ          1
+#define FILE_SHARE_WRITE         2
+#define FILE_SHARE_DELETE        4
+#define CREATE_NEW               1
+#define CREATE_ALWAYS            2
+#define OPEN_EXISTING            3
+#define OPEN_ALWAYS              4
+#define TRUNCATE_EXISTING        5
 #define FILE_BEGIN               0
 #define FILE_CURRENT             1
 #define FILE_END                 2
@@ -87,10 +192,73 @@ typedef struct _OVERLAPPED
   HANDLE hEvent;
 } OVERLAPPED, *POVERLAPPED, *LPOVERLAPPED;
 #include "poppack.h"
+/* ja:ファイル(属性) */
+#define FILE_ATTRIBUTE_READONLY            0x00000001
+#define FILE_ATTRIBUTE_HIDDEN              0x00000002
+#define FILE_ATTRIBUTE_SYSTEM              0x00000004
+#define FILE_ATTRIBUTE_DIRECTORY           0x00000010
+#define FILE_ATTRIBUTE_ARCHIVE             0x00000020
+#define FILE_ATTRIBUTE_NORMAL              0x00000080
+#define FILE_ATTRIBUTE_TEMPORARY           0x00000100
+#define FILE_ATTRIBUTE_SPARSE_FILE         0x00000200
+#define FILE_ATTRIBUTE_REPARSE_POINT       0x00000400
+#define FILE_ATTRIBUTE_COMPRESSED          0x00000800
+#define FILE_ATTRIBUTE_OFFLINE             0x00001000
+#define FILE_ATTRIBUTE_NOT_CONTENT_INDEXED 0x00002000
+#define FILE_ATTRIBUTE_ENCRYPTED           0x00004000
+/* ja:ファイル(メモリマップ) */
+#define PAGE_READONLY       0x00000002
+#define PAGE_READWRITE      0x00000004
+#define PAGE_WRITECOPY      0x00000008
+#define SEC_IMAGE           0x01000000
+#define SEC_RESERVE         0x04000000
+#define SEC_COMMIT          0x08000000
+#define SEC_NOCACHE         0x10000000
+#define FILE_MAP_COPY       1
+#define FILE_MAP_WRITE      2
+#define FILE_MAP_READ       4
+#define FILE_MAP_ALL_ACCESS 0x000f001f
 /* ja:ファイル(Win16互換) */
-#include "misc/fileio.h"
 #define HFILE_ERROR (GINT_TO_POINTER(-1))
 typedef FileIO *HFILE;
+/* ja:ファイル検索 */
+#include "pshpack1.h"
+typedef struct _WIN32_FIND_DATAA
+{
+  DWORD dwFileAttributes;
+  FILETIME ftCreateTime;
+  FILETIME ftLastAccessTime;
+  FILETIME ftLastWriteTime;
+  DWORD nFileSizeHigh;
+  DWORD nFileSizeLow;
+  DWORD dwReserved0;
+  DWORD dwReserved1;
+  CHAR cFileName[MAX_PATH];
+  CHAR cAlternateFileName[14];
+} WIN32_FIND_DATAA, *PWIN32_FIND_DATAA, *LPWIN32_FIND_DATAA;
+typedef struct _WIN32_FIND_DATAW
+{
+  DWORD dwFileAttributes;
+  FILETIME ftCreateTime;
+  FILETIME ftLastAccessTime;
+  FILETIME ftLastWriteTime;
+  DWORD nFileSizeHigh;
+  DWORD nFileSizeLow;
+  DWORD dwReserved0;
+  DWORD dwReserved1;
+  WCHAR cFileName[MAX_PATH];
+  WCHAR cAlternateFileName[14];
+} WIN32_FIND_DATAW, *PWIN32_FIND_DATAW, *LPWIN32_FIND_DATAW;
+#include "poppack.h"
+#ifdef UNICODE
+# define WIN32_FIND_DATA WIN32_FIND_DATAW
+# define PWIN32_FIND_DATA PWIN32_FIND_DATAW
+# define LPWIN32_FIND_DATA LPWIN32_FIND_DATAW
+#else /* not UNICODE */
+# define WIN32_FIND_DATA WIN32_FIND_DATAA
+# define PWIN32_FIND_DATA PWIN32_FIND_DATAA
+# define LPWIN32_FIND_DATA LPWIN32_FIND_DATAA
+#endif /* not UNICODE */
 /* ja:言語/ロケール/コードページ */
 #define MAX_DEFAULTCHAR 2
 #define MAX_LEADBYTES   12
@@ -141,7 +309,7 @@ typedef FileIO *HFILE;
 #define LANG_MAIN_GERMAN                 0x07
 #define LANG_MAIN_GREEK                  0x08
 #define LANG_MAIN_GUJARATI               0x47
-#define LANG_MAIN_HEBREW                 0x0D
+#define LANG_MAIN_HEBREW                 0x0d
 #define LANG_MAIN_HINDI                  0x39
 #define LANG_MAIN_HUNGARIAN              0x0e
 #define LANG_MAIN_ICELANDIC              0x0f
@@ -335,6 +503,7 @@ typedef struct
   UINT CodePage;
   WCHAR CodePageName[MAX_PATH];
 } CPINFOEXW, *LPCPINFOEXW;
+#include "poppack.h"
 #ifdef UNICODE
 # define CPINFOEX CPINFOEXW
 # define LPCPINFOEX LPCPINFOEXW
@@ -342,17 +511,65 @@ typedef struct
 # define CPINFOEX CPINFOEXA
 # define LPCPINFOEX LPCPINFOEXA
 #endif /* not UNICODE */
-#include "poppack.h"
 /* ja:環境 */
 #define VER_PLATFORM_WIN32s        0
 #define VER_PLATFORM_WIN32_WINDOWS 1
 #define VER_PLATFORM_WIN32_NT      2
-#ifdef G_HAVE_GINT64
-typedef gint64 LARGE_INTEGER, *PLARGE_INTEGER, *LPLARGE_INTEGER;
-#else /* not G_HAVE_GINT64 */
-typedef gint32 LARGE_INTEGER, *PLARGE_INTEGER, *LPLARGE_INTEGER;
-#endif /* not G_HAVE_GINT64 */
 #include "pshpack1.h"
+typedef union _LARGE_INTEGER
+{
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+  struct
+  {
+    DWORD LowPart;
+    LONG  HighPart;
+  };
+  struct
+  {
+    DWORD LowPart;
+    LONG  HighPart;
+  } u;
+#else /* not G_BYTE_ORDER == G_LITTLE_ENDIAN */
+  struct
+  {
+    LONG  HighPart;
+    DWORD LowPart;
+  };
+  struct
+  {
+    LONG  HighPart;
+    DWORD LowPart;
+  } u;
+#endif /* not G_BYTE_ORDER == G_LITTLE_ENDIAN */
+  LONGLONG QuadPart;
+} LARGE_INTEGER, *PLARGE_INTEGER, *LPLARGE_INTEGER;
+typedef union _ULARGE_INTEGER
+{
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+  struct
+  {
+    DWORD LowPart;
+    DWORD HighPart;
+  };
+  struct
+  {
+    DWORD LowPart;
+    DWORD HighPart;
+  } u;
+#else /* not G_BYTE_ORDER == G_LITTLE_ENDIAN */
+  struct
+  {
+    DWORD HighPart;
+    DWORD LowPart;
+  };
+  struct
+  {
+    DWORD HighPart;
+    DWORD LowPart;
+  } u;
+#endif /* not G_BYTE_ORDER == G_LITTLE_ENDIAN */
+  ULONGLONG QuadPart;
+} ULARGE_INTEGER, *PULARGE_INTEGER, *LPULARGE_INTEGER;
 typedef struct
 {
   DWORD dwOSVersionInfoSize;
@@ -371,6 +588,7 @@ typedef struct
   DWORD dwPlatformId;
   WCHAR szCSDVersion[128];
 } OSVERSIONINFOW, *POSVERSIONINFOW, *LPOSVERSIONINFOW;
+#include "poppack.h"
 #ifdef UNICODE
 # define OSVERSIONINFO OSVERSIONINFOW
 # define POSVERSIONINFO POSVERSIONINFOW
@@ -380,55 +598,7 @@ typedef struct
 # define POSVERSIONINFO POSVERSIONINFOA
 # define LPOSVERSIONINFO LPOSVERSIONINFOA
 #endif /* not UNICODE */
-typedef struct _STARTUPINFOA
-{
-  DWORD cb;
-  LPSTR lpReserved;
-  LPSTR lpDesktop;
-  LPSTR lpTitle;
-  DWORD dwX;
-  DWORD dwY;
-  DWORD dwXSize;
-  DWORD dwYSize;
-  DWORD dwXCountChars;
-  DWORD dwYCountChars;
-  DWORD dwFillAttribute;
-  DWORD dwFlags;
-  WORD wShowWindow;
-  WORD cbReserved2;
-  BYTE *lpReserved2;
-  HANDLE hStdInput;
-  HANDLE hStdOutput;
-  HANDLE hStdError;
-} STARTUPINFOA, *LPSTARTUPINFOA;
-typedef struct _STARTUPINFOW
-{
-  DWORD cb;
-  LPWSTR lpReserved;
-  LPWSTR lpDesktop;
-  LPWSTR lpTitle;
-  DWORD dwX;
-  DWORD dwY;
-  DWORD dwXSize;
-  DWORD dwYSize;
-  DWORD dwXCountChars;
-  DWORD dwYCountChars;
-  DWORD dwFillAttribute;
-  DWORD dwFlags;
-  WORD wShowWindow;
-  WORD cbReserved2;
-  BYTE *lpReserved2;
-  HANDLE hStdInput;
-  HANDLE hStdOutput;
-  HANDLE hStdError;
-} STARTUPINFOW, *LPSTARTUPINFOW;
-#ifdef UNICODE
-# define STARTUPINFO STARTUPINFOW
-# define LPSTARTUPINFO LPSTARTUPINFOW
-#else /* not UNICODE */
-# define STARTUPINFO STARTUPINFOA
-# define LPSTARTUPINFO LPSTARTUPINFOA
-#endif /* not UNICODE */
+#include "pshpack1.h"
 typedef struct _SYSTEM_INFO
 {
   union {
@@ -467,15 +637,29 @@ typedef struct tagCOORD
   SHORT X;
   SHORT Y;
 } COORD, *PCOORD, *LPCOORD;
+/* ja:非公開 */
+#define EXCEPTION_MAXIMUM_PARAMETERS 15
+typedef struct _EXCEPTION_RECORD
+{
+  DWORD ExceptionCode;
+  DWORD ExceptionFlags;
+  struct _EXCEPTION_RECORD *ExceptionRecord;
+  PVOID ExceptionAddress;
+  DWORD NumberParameters;
+  ULONG_PTR ExceptionInformation[EXCEPTION_MAXIMUM_PARAMETERS];
+} EXCEPTION_RECORD, *PEXCEPTION_RECORD, *LPEXCEPTION_RECORD;
 #include "poppack.h"
 
 
 extern W32LdrExport kernel32_exports[];
 
 
+ATOM WINAPI AddAtomA (LPCSTR lpString);
+ATOM WINAPI AddAtomW (LPCWSTR lpString);
 BOOL WINAPI AllocConsole (VOID);
 BOOL WINAPI CloseHandle (HANDLE hObject);
-BOOL WINAPI CopyFileA (LPCSTR lpExistingFileName, LPCWSTR lpNewFileName, BOOL bFailIfExists);
+LONG WINAPI CompareFileTime (CONST LPFILETIME lpFileTime1, CONST LPFILETIME lpFileTime2);
+BOOL WINAPI CopyFileA (LPCSTR lpExistingFileName, LPCSTR lpNewFileName, BOOL bFailIfExists);
 BOOL WINAPI CopyFileW (LPCWSTR lpExistingFileName, LPCWSTR lpNewFileName, BOOL bFailIfExists);
 BOOL WINAPI CreateDirectoryA (LPCSTR lpPathName, LPSECURITY_ATTRIBUTES lpSecurityAttributes);
 BOOL WINAPI CreateDirectoryW (LPCWSTR lpPathName, LPSECURITY_ATTRIBUTES lpSecurityAttributes);
@@ -485,9 +669,14 @@ HANDLE WINAPI CreateFileA (LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwSha
 HANDLE WINAPI CreateFileMappingA (HANDLE hFile, LPSECURITY_ATTRIBUTES lpAttributes, DWORD flProtect, DWORD dwMaximumSizeHigh, DWORD dwMaximumSizeLow, LPCSTR lpName);
 HANDLE WINAPI CreateFileMappingW (HANDLE hFile, LPSECURITY_ATTRIBUTES lpAttributes, DWORD flProtect, DWORD dwMaximumSizeHigh, DWORD dwMaximumSizeLow, LPCWSTR lpName);
 HANDLE WINAPI CreateFileW (LPCWSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile);
+HANDLE WINAPI CreateMutexA (LPSECURITY_ATTRIBUTES lpMutexAttributes, BOOL bInitialOwner, LPCSTR lpName);
+HANDLE WINAPI CreateMutexW (LPSECURITY_ATTRIBUTES lpMutexAttributes, BOOL bInitialOwner, LPCWSTR lpName);
+BOOL WINAPI CreateProcessA (LPCSTR lpApplicationName, LPSTR lpCommandLine, LPSECURITY_ATTRIBUTES lpProcessAttributes, LPSECURITY_ATTRIBUTES lpThreadAttributes, BOOL bInheritHandles, DWORD dwCreationFlags, LPVOID lpEnvironment, LPCSTR lpCurrentDirectory, LPSTARTUPINFOA lpStartupInfo, LPPROCESS_INFORMATION lpProcessInformation);
+BOOL WINAPI CreateProcessW (LPCWSTR lpApplicationName, LPWSTR lpCommandLine, LPSECURITY_ATTRIBUTES lpProcessAttributes, LPSECURITY_ATTRIBUTES lpThreadAttributes, BOOL bInheritHandles, DWORD dwCreationFlags, LPVOID lpEnvironment, LPCWSTR lpCurrentDirectory, LPSTARTUPINFOW lpStartupInfo, LPPROCESS_INFORMATION lpProcessInformation);
 HANDLE WINAPI CreateSemaphoreA (LPSECURITY_ATTRIBUTES lpSemaphoreAttributes, LONG lInitialCount, LONG lMaximumCount, LPCSTR lpName);
 HANDLE WINAPI CreateSemaphoreW (LPSECURITY_ATTRIBUTES lpSemaphoreAttributes, LONG lInitialCount, LONG lMaximumCount, LPCWSTR lpName);
 HANDLE WINAPI CreateThread (LPSECURITY_ATTRIBUTES lpThreadAttributes, DWORD dwStackSize, LPTHREAD_START_ROUTINE lpStartAddress, LPVOID lpParameter, DWORD dwCreationFlags, LPDWORD lpThreadId);
+ATOM WINAPI DeleteAtom (ATOM nAtom);
 VOID WINAPI DeleteCriticalSection (LPCRITICAL_SECTION lpCriticalSection);
 BOOL WINAPI DeleteFileA (LPCSTR lpFileName);
 BOOL WINAPI DeleteFileW (LPCWSTR lpFileName);
@@ -496,6 +685,16 @@ VOID WINAPI EnterCriticalSection (LPCRITICAL_SECTION lpCriticalSection);
 BOOL WINAPI EnumCalendarInfoA (CALINFO_ENUMPROCA lpCalInfoEnumProc, LCID Locale, CALID Calendar, CALTYPE CalType);
 BOOL WINAPI EnumCalendarInfoW (CALINFO_ENUMPROCW lpCalInfoEnumProc, LCID Locale, CALID Calendar, CALTYPE CalType);
 VOID WINAPI ExitProcess (UINT uExitCode);
+VOID WINAPI ExitThread (UINT uExitCode);
+BOOL WINAPI FileTimeToLocalFileTime (CONST LPFILETIME lpFileTime, LPFILETIME lpLocalFileTime);
+BOOL WINAPI FileTimeToSystemTime (CONST LPFILETIME lpFileTime, LPSYSTEMTIME lpSystemTime);
+ATOM WINAPI FindAtomA (LPCSTR lpString);
+ATOM WINAPI FindAtomW (LPCWSTR lpString);
+BOOL WINAPI FindClose (HANDLE hFind);
+HANDLE WINAPI FindFirstFileA (LPCSTR lpFileName, LPWIN32_FIND_DATAA lpFindFileData);
+HANDLE WINAPI FindFirstFileW (LPCWSTR lpFileName, LPWIN32_FIND_DATAW lpFindFileData);
+BOOL WINAPI FindNextFileA (HANDLE hFind, LPWIN32_FIND_DATAA lpFindFileData);
+BOOL WINAPI FindNextFileW (HANDLE hFind, LPWIN32_FIND_DATAW lpFindFileData);
 HRSRC WINAPI FindResourceA (HMODULE hModule, LPCSTR lpName, LPCSTR lpType);
 HRSRC WINAPI FindResourceExA (HMODULE hModule, LPCSTR lpType, LPCSTR lpName, WORD wLanguage);
 HRSRC WINAPI FindResourceExW (HMODULE hModule, LPCWSTR lpType, LPCWSTR lpName, WORD wLanguage);
@@ -506,24 +705,37 @@ BOOL WINAPI FreeEnvironmentStringsW (LPWSTR lpszEnvironmentBlock);
 BOOL WINAPI FreeLibrary (HMODULE hModule);
 BOOL WINAPI FreeResource (HGLOBAL hResData);
 UINT WINAPI GetACP (VOID);
+UINT WINAPI GetAtomNameA (ATOM nAtom, LPSTR lpBuffer, INT nSize);
+UINT WINAPI GetAtomNameW (ATOM nAtom, LPWSTR lpBuffer, INT nSize);
 BOOL WINAPI GetCPInfo (UINT CodePage, LPCPINFO lpCPInfo);
 BOOL WINAPI GetCPInfoExA (UINT CodePage, DWORD dwFlags, LPCPINFOEXA lpCPInfoEx);
 BOOL WINAPI GetCPInfoExW (UINT CodePage, DWORD dwFlags, LPCPINFOEXW lpCPInfoEx);
 LPSTR WINAPI GetCommandLineA (VOID);
 LPWSTR WINAPI GetCommandLineW (VOID);
+DWORD WINAPI GetCurrentDirectoryA (DWORD nBufferLength, LPSTR lpBuffer);
+DWORD WINAPI GetCurrentDirectoryW (DWORD nBufferLength, LPWSTR lpBuffer);
 HANDLE WINAPI GetCurrentProcess (VOID);
 HANDLE WINAPI GetCurrentThread (VOID);
 DWORD WINAPI GetCurrentThreadId (VOID);
+BOOL GetDiskFreeSpaceA (LPCSTR lpRootPathName, LPDWORD lpSectorsPerCluster, LPDWORD lpBytesPerSector, LPDWORD lpNumberOfFreeClusters, LPDWORD lpTotalNumberOfClusters);
+BOOL WINAPI GetDiskFreeSpaceExA (LPCSTR lpDirectoryName, PULARGE_INTEGER lpFreeBytesAvailableToCaller, PULARGE_INTEGER lpTotalNumberOfBytes, PULARGE_INTEGER lpTotalNumberOfFreeBytes);
+BOOL WINAPI GetDiskFreeSpaceExW (LPCWSTR lpDirectoryName, PULARGE_INTEGER lpFreeBytesAvailableToCaller, PULARGE_INTEGER lpTotalNumberOfBytes, PULARGE_INTEGER lpTotalNumberOfFreeBytes);
+BOOL WINAPI GetDiskFreeSpaceW (LPCWSTR lpRootPathName, LPDWORD lpSectorsPerCluster, LPDWORD lpBytesPerSector, LPDWORD lpNumberOfFreeClusters, LPDWORD lpTotalNumberOfClusters);
 LPVOID WINAPI GetEnvironmentStringsA (VOID);
 LPVOID WINAPI GetEnvironmentStringsW (VOID);
 DWORD WINAPI GetEnvironmentVariableA (LPCSTR lpName, LPSTR lpBuffer, DWORD nSize);
 DWORD WINAPI GetEnvironmentVariableW (LPCWSTR lpName, LPWSTR lpBuffer, DWORD nSize);
+DWORD WINAPI GetFileAttributesA (LPCSTR lpFileName);
+DWORD WINAPI GetFileAttributesW (LPCWSTR lpFileName);
 DWORD WINAPI GetFileSize (HANDLE hFile, LPDWORD lpFileSizeHigh);
+BOOL WINAPI GetFileTime (HANDLE hFile, LPFILETIME lpCreationTime, LPFILETIME lpLastAccessTime, LPFILETIME lpLastWriteTime);
 DWORD WINAPI GetFileType (HANDLE hFile);
 DWORD WINAPI GetLastError (VOID);
 VOID WINAPI GetLocalTime (LPSYSTEMTIME lpSystemTime);
 INT WINAPI GetLocaleInfoA (LCID Locale, LCTYPE LCType, LPSTR lpLCData, INT cchData);
 INT WINAPI GetLocaleInfoW (LCID Locale, LCTYPE LCType, LPWSTR lpLCData, INT cchData);
+DWORD WINAPI GetLogicalDriveStringsA (DWORD nBufferLength, LPSTR lpBuffer);
+DWORD WINAPI GetLogicalDriveStringsW (DWORD nBufferLength, LPWSTR lpBuffer);
 DWORD WINAPI GetModuleFileNameA (HMODULE hModule, LPSTR lpFilename, DWORD nSize);
 DWORD WINAPI GetModuleFileNameW (HMODULE hModule, LPWSTR lpFilename, DWORD nSize);
 HMODULE WINAPI GetModuleHandleA (LPCSTR lpModuleName);
@@ -557,6 +769,8 @@ LANGID WINAPI GetSystemDefaultLangID (VOID);
 LANGID WINAPI GetSystemDefaultUILanguage (VOID);
 UINT WINAPI GetSystemDirectoryA (LPSTR lpBuffer, UINT uSize);
 UINT WINAPI GetSystemDirectoryW (LPWSTR lpBuffer, UINT uSize);
+DWORD WINAPI GetTempPathA (DWORD nBufferLength, LPSTR lpBuffer);
+DWORD WINAPI GetTempPathW (DWORD nBufferLength, LPWSTR lpBuffer);
 LCID WINAPI GetThreadLocale (VOID);
 DWORD WINAPI GetUserDefaultLCID (VOID);
 LANGID WINAPI GetUserDefaultLangID (VOID);
@@ -564,9 +778,16 @@ LANGID WINAPI GetUserDefaultUILanguage (VOID);
 DWORD WINAPI GetVersion (VOID);
 BOOL WINAPI GetVersionExA (LPOSVERSIONINFOA lpVersionInfo);
 BOOL WINAPI GetVersionExW (LPOSVERSIONINFOW lpVersionInfo);
+ATOM WINAPI GlobalAddAtomA (LPCSTR lpString);
+ATOM WINAPI GlobalAddAtomW (LPCWSTR lpString);
 HGLOBAL WINAPI GlobalAlloc (UINT uFlags, UINT uBytes);
+ATOM WINAPI GlobalDeleteAtom (ATOM nAtom);
+ATOM WINAPI GlobalFindAtomA (LPCSTR lpString);
+ATOM WINAPI GlobalFindAtomW (LPCWSTR lpString);
 UINT WINAPI GlobalFlags (HGLOBAL hMem);
 HGLOBAL WINAPI GlobalFree (HGLOBAL hMem);
+UINT WINAPI GlobalGetAtomNameA (ATOM nAtom, LPSTR lpBuffer, INT nSize);
+UINT WINAPI GlobalGetAtomNameW (ATOM nAtom, LPWSTR lpBuffer, INT nSize);
 HGLOBAL WINAPI GlobalHandle (LPCVOID pMem);
 LPVOID WINAPI GlobalLock (HGLOBAL hMem);
 HGLOBAL WINAPI GlobalReAlloc (HGLOBAL hMem, UINT uBytes, UINT uFlags);
@@ -578,6 +799,7 @@ BOOL WINAPI HeapDestroy (HANDLE hHeap);
 BOOL WINAPI HeapFree (HANDLE hHeap, DWORD dwFlags, LPVOID lpMem);
 LPVOID WINAPI HeapReAlloc (HANDLE hHeap, DWORD dwFlags, LPVOID lpMem, DWORD dwBytes);
 DWORD WINAPI HeapSize (HANDLE hHeap, DWORD dwFlags, LPCVOID lpMem);
+BOOL WINAPI InitAtomTable (DWORD nSize);
 VOID WINAPI InitializeCriticalSection (LPCRITICAL_SECTION lpCriticalSection);
 PVOID WINAPI InterlockedCompareExchange (PVOID *Destination, PVOID Exchange, PVOID Comperand);
 LONG WINAPI InterlockedDecrement (LPLONG lpAddend);
@@ -598,6 +820,7 @@ HINSTANCE WINAPI LoadLibraryExW (LPCWSTR lpLibFileName, HANDLE hFile, DWORD dwFl
 HINSTANCE WINAPI LoadLibraryW (LPCWSTR lpFileName);
 HGLOBAL WINAPI LoadResource (HMODULE hModule, HRSRC hResInfo);
 HLOCAL WINAPI LocalAlloc (UINT uFlags, UINT uBytes);
+BOOL WINAPI LocalFileTimeToFileTime (CONST LPFILETIME lpLocalFileTime, LPFILETIME lpFileTime);
 UINT WINAPI LocalFlags (HLOCAL hMem);
 HLOCAL WINAPI LocalFree (HLOCAL hMem);
 HLOCAL WINAPI LocalHandle (LPCVOID pMem);
@@ -614,25 +837,39 @@ BOOL WINAPI MoveFileExW (LPCWSTR lpExistingFileName, LPCWSTR lpNewFileName, DWOR
 BOOL WINAPI MoveFileW (LPCWSTR lpExistingFileName, LPCWSTR lpNewFileName);
 INT WINAPI MulDiv (INT nNumber, INT nNumerator, INT nDenominator);
 INT WINAPI MultiByteToWideChar (UINT CodePage, DWORD dwFlags, LPCSTR lpMultiByteStr, INT cchMultiByte, LPWSTR lpWideCharStr, INT cchWideChar);
+HANDLE WINAPI OpenEventA (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCSTR lpName);
+HANDLE WINAPI OpenEventW (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCWSTR lpName);
 HANDLE WINAPI OpenFileMappingA (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCSTR lpName);
 HANDLE WINAPI OpenFileMappingW (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCWSTR lpName);
+HANDLE WINAPI OpenMutexA (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCSTR lpName);
+HANDLE WINAPI OpenMutexW (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCWSTR lpName);
+HANDLE WINAPI OpenSemaphoreA (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCSTR lpName);
+HANDLE WINAPI OpenSemaphoreW (DWORD dwDesiredAccess, BOOL bInheritHandle, LPCWSTR lpName);
 VOID WINAPI OutputDebugStringA (LPCSTR lpOutputString);
 VOID WINAPI OutputDebugStringW (LPCWSTR lpOutputString);
+BOOL WINAPI PulseEvent (HANDLE hEvent);
 BOOL WINAPI QueryPerformanceCounter (LARGE_INTEGER *lpPerformanceCount);
 BOOL WINAPI QueryPerformanceFrequency (LARGE_INTEGER *lpFrequency);
 BOOL WINAPI ReadFile (HANDLE hFile, LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead, LPOVERLAPPED lpOverlapped);
+BOOL WINAPI ReleaseMutex (HANDLE hMutex);
 BOOL WINAPI ReleaseSemaphore (HANDLE hSemaphore, LONG lReleaseCount, LPLONG lpPreviousCount);
 BOOL WINAPI RemoveDirectoryA (LPCSTR lpPathName);
 BOOL WINAPI RemoveDirectoryW (LPCWSTR lpPathName);
 BOOL WINAPI ResetEvent (HANDLE hEvent);
+VOID WINAPI RtlUnwind (PVOID TargetFrame, PVOID TargetIp, PEXCEPTION_RECORD ExceptionRecord, PVOID ReturnValue);
 BOOL WINAPI SetConsoleScreenBufferSize (HANDLE hConsoleOutput, COORD dwSize);
+BOOL WINAPI SetCurrentDirectoryA (LPCSTR lpPathName);
+BOOL WINAPI SetCurrentDirectoryW (LPCWSTR lpPathName);
 BOOL WINAPI SetEndOfFile (HANDLE hFile);
+BOOL WINAPI SetEvent (HANDLE hEvent);
 DWORD WINAPI SetFilePointer (HANDLE hFile, LONG lDistanceToMove, PLONG lpDistanceToMoveHigh, DWORD dwMoveMethod);
+BOOL WINAPI SetFileTime (HANDLE hFile, CONST LPFILETIME lpCreationTime, CONST LPFILETIME lpLastAccessTime, CONST LPFILETIME lpLastWriteTime);
 UINT WINAPI SetHandleCount (UINT uNumber);
 VOID WINAPI SetLastError (DWORD dwErrCode);
 BOOL WINAPI SetStdHandle (DWORD nStdHandle, HANDLE hHandle);
 DWORD WINAPI SizeofResource (HMODULE hModule, HRSRC hResInfo);
 VOID WINAPI Sleep (DWORD dwMilliseconds);
+BOOL WINAPI SystemTimeToFileTime (CONST LPSYSTEMTIME lpSystemTime, LPFILETIME lpFileTime);
 BOOL WINAPI TerminateProcess (HANDLE hProcess, UINT uExitCode);
 DWORD WINAPI TlsAlloc (VOID);
 BOOL WINAPI TlsFree (DWORD dwTlsIndex);
@@ -666,6 +903,7 @@ BOOL WINAPI WriteProfileStringW (LPCWSTR lpAppName, LPCWSTR lpKeyName, LPCWSTR l
 HFILE WINAPI _lclose (HFILE hFile);
 HFILE WINAPI _lcreat (LPCSTR lpPathName, INT iAttribute);
 LONG WINAPI _llseek (HFILE hFile, LONG lOffset, INT iOrigin);
+HFILE WINAPI _lopen (LPCSTR lpPathName, INT iReadWrite);
 UINT WINAPI _lread (HFILE hFile, LPVOID lpBuffer, UINT uBytes);
 UINT WINAPI _lwrite (HFILE hFile, LPVOID lpBuffer, UINT uBytes);
 LPSTR WINAPI lstrcatA (LPSTR lpString1, LPCSTR lpString2);
@@ -683,21 +921,33 @@ INT WINAPI lstrlenW (LPCWSTR lpString);
 
 
 #ifdef UNICODE
+# define AddAtom AddAtomW
 # define CopyFile CopyFileW
 # define CreateDirectory CreateDirectoryW
 # define CreateEvent CreateEventW
 # define CreateFile CreateFileW
 # define CreateFileMapping CreateFileMappingW
+# define CreateMutex CreateMutexW
+# define CreateProcess CreateProcessW
 # define CreateSemaphore CreateSemaphoreW
 # define DeleteFile DeleteFileW
 # define EnumCalendarInfo EnumCalendarInfoW
+# define FindAtom FindAtomW
+# define FindFirstFile FindFirstFileW
+# define FindNextFile FindNextFileW
 # define FindResource FindResourceW
 # define FindResourceEx FindResourceExW
 # define FreeEnvironmentStrings FreeEnvironmentStringsW
+# define GetAtomName GetAtomNameW
 # define GetCPInfoEx GetCPInfoExW
 # define GetCommandLine GetCommandLineW
+# define GetCurrentDirectory GetCurrentDirectoryW
+# define GetDiskFreeSpace GetDiskFreeSpaceW
+# define GetDiskFreeSpaceEx GetDiskFreeSpaceExW
 # define GetEnvironmentStrings GetEnvironmentStringsW
 # define GetEnvironmentVariable GetEnvironmentVariableW
+# define GetFileAttributes GetFileAttributesW
+# define GetLogicalDriveStrings GetLogicalDriveStringsW
 # define GetModuleFileName GetModuleFileNameW
 # define GetModuleHandle GetModuleHandleW
 # define GetPrivateProfileInt GetPrivateProfileIntW
@@ -710,15 +960,23 @@ INT WINAPI lstrlenW (LPCWSTR lpString);
 # define GetShortPathName GetShortPathNameW
 # define GetStringType GetStringTypeW
 # define GetSystemDirectory GetSystemDirectoryW
+# define GetTempPath GetTempPathW
 # define GetVersionEx GetVersionExW
+# define GlobalAddAtom GlobalAddAtomW
+# define GlobalFindAtom GlobalFindAtomW
+# define GlobalGetAtomName GlobalGetAtomNameW
 # define LCMapString LCMapStringW
 # define LoadLibrary LoadLibraryW
 # define LoadLibraryEx LoadLibraryExW
 # define MoveFile MoveFileW
 # define MoveFileEx MoveFileExW
+# define OpenEvent OpenEventW
 # define OpenFileMapping OpenFileMappingW
+# define OpenMutex OpenMutexW
+# define OpenSemaphore OpenSemaphoreW
 # define OutputDebugString OutputDebugStringW
 # define RemoveDirectory RemoveDirectoryW
+# define SetCurrentDirectory SetCurrentDirectoryW
 # define VerLanguageName VerLanguageNameW
 # define WriteConsole WriteConsoleW
 # define WritePrivateProfileSection WritePrivateProfileSectionW
@@ -733,21 +991,33 @@ INT WINAPI lstrlenW (LPCWSTR lpString);
 # define lstrcpyn lstrcpynW
 # define lstrlen lstrlenW
 #else /* not UNICODE*/
+# define AddAtom AddAtomA
 # define CopyFile CopyFileA
 # define CreateDirectory CreateDirectoryA
 # define CreateEvent CreateEventA
 # define CreateFile CreateFileA
 # define CreateFileMapping CreateFileMappingA
+# define CreateMutex CreateMutexA
+# define CreateProcess CreateProcessA
 # define CreateSemaphore CreateSemaphoreA
 # define DeleteFile DeleteFileA
 # define EnumCalendarInfo EnumCalendarInfoA
+# define FindAtom FindAtomA
+# define FindFirstFile FindFirstFileA
+# define FindNextFile FindNextFileA
 # define FindResource FindResourceA
 # define FindResourceEx FindResourceExA
 # define FreeEnvironmentStrings FreeEnvironmentStringsA
+# define GetAtomName GetAtomNameA
 # define GetCPInfoEx GetCPInfoExA
 # define GetCommandLine GetCommandLineA
+# define GetCurrentDirectory GetCurrentDirectoryA
+# define GetDiskFreeSpace GetDiskFreeSpaceA
+# define GetDiskFreeSpaceEx GetDiskFreeSpaceExA
 # define GetEnvironmentStrings GetEnvironmentStringsA
 # define GetEnvironmentVariable GetEnvironmentVariableA
+# define GetFileAttributes GetFileAttributesA
+# define GetLogicalDriveStrings GetLogicalDriveStringsA
 # define GetModuleFileName GetModuleFileNameA
 # define GetModuleHandle GetModuleHandleA
 # define GetPrivateProfileInt GetPrivateProfileIntA
@@ -760,15 +1030,23 @@ INT WINAPI lstrlenW (LPCWSTR lpString);
 # define GetShortPathName GetShortPathNameA
 # define GetStringType GetStringTypeA
 # define GetSystemDirectory GetSystemDirectoryA
+# define GetTempPath GetTempPathA
 # define GetVersionEx GetVersionExA
+# define GlobalAddAtom GlobalAddAtomA
+# define GlobalFindAtom GlobalFindAtomA
+# define GlobalGetAtomName GlobalGetAtomNameA
 # define LCMapString LCMapStringA
 # define LoadLibrary LoadLibraryA
 # define LoadLibraryEx LoadLibraryExA
 # define MoveFile MoveFileA
 # define MoveFileEx MoveFileExA
+# define OpenEvent OpenEventA
 # define OpenFileMapping OpenFileMappingA
+# define OpenMutex OpenMutexA
+# define OpenSemaphore OpenSemaphoreA
 # define OutputDebugString OutputDebugStringA
 # define RemoveDirectory RemoveDirectoryA
+# define SetCurrentDirectory SetCurrentDirectoryA
 # define VerLanguageName VerLanguageNameA
 # define WriteConsole WriteConsoleA
 # define WritePrivateProfileSection WritePrivateProfileSectionA
