@@ -1,6 +1,6 @@
 /*
     w32loader
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #define __WINDEF_H__
 
 
-#include "common.h"
+#include "gcommon.h"
 
 
 #ifdef __cplusplus
@@ -35,6 +35,8 @@ extern "C" {
 #define WINAPI __stdcall
 #define CDECL __cdecl
 #define WINAPIV __cdecl
+#define NEAR
+#define FAR
 
 
 /* en:Macros to split words and longs. */
@@ -55,20 +57,26 @@ extern "C" {
 
 /* ja:基本 */
 #define CONST const
-typedef gint32      BOOL,   *PBOOL,   *LPBOOL;
-typedef void        VOID,   *PVOID,   *LPVOID;
-typedef const void  CVOID,  *PCVOID,  *LPCVOID;
-typedef gchar       CHAR,   *PCHAR,   *LPCHAR,   *CHAR_PTR;
-typedef guchar      UCHAR,  *PUCHAR,  *LPUCHAR,  *UCHAR_PTR;
-typedef gunichar2   WCHAR,  *PWCHAR,  *LPWCHAR,  *WCHAR_PTR;
-typedef gunichar2   WUCHAR, *PWUCHAR, *LPWUCHAR, *WUCHAR_PTR;
-typedef gint16      SHORT,  *PSHORT,  *LPSHORT,  *SHORT_PTR;
-typedef guint16     USHORT, *PUSHORT, *LPUSHORT, *USHORT_PTR;
-typedef gint32      INT,    *PINT,    *LPINT,    *INT_PTR;
-typedef guint32     UINT,   *PUINT,   *LPUINT,   *UINT_PTR;
-typedef gint32      LONG,   *PLONG,   *LPLONG,   *LONG_PTR;
-typedef guint32     ULONG,  *PULONG,  *LPULONG,  *ULONG_PTR;
+typedef gint32      BOOL,      *PBOOL,      *LPBOOL;
+typedef void        VOID,      *PVOID,      *LPVOID;
+typedef const void  CVOID,     *PCVOID,     *LPCVOID;
+typedef gchar       CHAR,      *PCHAR,      *LPCHAR;
+typedef guchar      UCHAR,     *PUCHAR,     *LPUCHAR;
+typedef gunichar2   WCHAR,     *PWCHAR,     *LPWCHAR;
+typedef gunichar2   WUCHAR,    *PWUCHAR,    *LPWUCHAR;
+typedef gint16      SHORT,     *PSHORT,     *LPSHORT;
+typedef guint16     USHORT,    *PUSHORT,    *LPUSHORT;
+typedef gint32      INT,       *PINT,       *LPINT;
+typedef guint32     UINT,      *PUINT,      *LPUINT;
+typedef gint32      LONG,      *PLONG,      *LPLONG;
+typedef guint32     ULONG,     *PULONG,     *LPULONG;
 typedef guint32     SIZE_T;
+typedef gint32      INT_PTR,   *PINT_PTR,   *LPINT_PTR;
+typedef guint32     UINT_PTR,  *PUINT_PTR,  *LPUINT_PTR;
+typedef gint32      LONG_PTR,  *PLONG_PTR,  *LPLONG_PTR;
+typedef guint32     ULONG_PTR, *PULONG_PTR, *LPULONG_PTR;
+typedef glonglong   LONGLONG,  *PLONGLONG,  *LPLONGLONG;
+typedef gulonglong  ULONGLONG, *PULONGLONG, *LPULONGLONG;
 /* en:WORD */
 typedef guint8      BYTE,   *PBYTE,   *LPBYTE;
 typedef guint16     WORD,   *PWORD,   *LPWORD;
@@ -87,6 +95,8 @@ typedef struct _W32LdrHandle *HANDLE;
 typedef struct _W32LdrModule *HINSTANCE, *HMODULE;
 typedef GtkWidget *HWND;
 typedef LPVOID HDC;
+/* ja:アトム */
+typedef WORD ATOM, *PATOM, *LPATOM;
 /* ja:構造体 */
 #include "pshpack1.h"
 typedef struct _SECURITY_ATTRIBUTES
@@ -95,6 +105,12 @@ typedef struct _SECURITY_ATTRIBUTES
   LPVOID lpSecurityDescriptor;
   BOOL bInheritHandle;
 } SECURITY_ATTRIBUTES, *PSECURITY_ATTRIBUTES, *LPSECURITY_ATTRIBUTES;
+/* ja:時間 */
+typedef struct _FILETIME
+{
+  DWORD dwLowDateTime;
+  DWORD dwHighDateTime;
+} FILETIME, *PFILETIME, *LPFILETIME;
 #include "poppack.h"
 
 
